###########################################################################
#
# ConvertingForexVolumeIntoDollars.R
#
# This script was used to perform the correlations and plot the charts used 
# in the article - "Converting Forex Volume Into Dollars"
#
# Copyright June 2021 by Tim Wilson www.MindYourTrading.com
#  
#
############################################################################ 

##  Set this line to point to the R script directory on your own PC
setwd( "C:/Users/Toshiba/Documents/MFTT/AnalysisRCode/OrderFlow")

## R include libraries
library(corrplot)
library(lubridate)
library(quantmod)
library(tseries)
library(ggpubr)

# Set default time zone- New York Close chart time (UTC +3)
Sys.setenv(TZ='Etc/GMT-3')

glvDataFilePath<- "C:/Users/Toshiba/Documents/MindYourTrading/ForexFactoryStuff/Data/"
glvFXSpotDataFile1 <- "PepperstoneGBPUSD1.csv"
glvFXSpotDataFile2 <- "GoMarketsGBPUSD1.csv"
glvFXSpotDataFile3 <- "BlackBullGBPUSD1.csv"
glvFXSpotDataFile4 <- "DukasCopyGBPUSD1.csv"
glvFXFuturesDataFile <- "6B0621Last.csv"  # FX Futures

# date times are chart time (UTC + 3)
glvFromDateTime <- "2021-05-31 00:00:00"  
glvToDateTime   <- "2021-06-01 23:59:00"

glvDataFilePath<- "C:/Users/Toshiba/Documents/MindYourTrading/ForexFactoryStuff/Data/"

# Dukas copy shift time in hrs to convert from GMT to New York Close chart time (UTC +3)
glvDukasCopyShiftHrs <- 3


# Most commonly used methods for calculating correlations
#========================================================
#
# method = "pearson"  -  best for normal distribution
# method = "kendal"   -  rank correlation - preferred for small samples with some outliers
# method = "spearman" -  rank correlation - ignores data distribution entirely


GetFXSpotData <- function(filePath,filename, fromDateTime, toDateTime)
  # Read and select data from a csv file for a time period
  # Return data in xts format
  # 
  #
  ###########################################################################
{
  # Create the full csv import file path and name which includes the currency symbol
  # and the time frame period of the imported data file.
  
  filename = paste(filePath, filename, sep = "")
  
  # Open and read the file.
  csvFileData <-lapply(Sys.glob(filename), read.csv,na.strings = "unknown")
  
  
  #Convert file csv data to a data frame.
  currencyData.df <- as.data.frame(csvFileData, stringAsFactors=FALSE)
  
  # Add headers.
  colHeadings <-  c('date','time', 'open', 'high', 'low', 'close', 'volume')
  names(currencyData.df) <- colHeadings
  
  
  # convert currency data to XTS
  # Extract datetimes as POSIXct date time from hourly currency Data
  # dateTime <- paste(currencyData.df$date,currencyData.df$time)
  dateTimes <- paste(currencyData.df$date, currencyData.df$time, sep = " ")
  posixDateTimes <- as.POSIXct(dateTimes,format= "%Y.%m.%d %H:%M")
  # Convert values and dates to an xts object
  currencyData.xts <- as.xts(cbind(currencyData.df['open'],currencyData.df['high'],
                                   currencyData.df['low'], currencyData.df['close'],
                                   currencyData.df['volume']),order.by = posixDateTimes)
  
  # convert start and end date times to POSIXct
  startDateTime <- as.POSIXct(fromDateTime,format= "%Y-%m-%d %H:%M:%S")
  endDateTime <- as.POSIXct(toDateTime,format= "%Y-%m-%d %H:%M:%S")
  
  # Select currency data for the date range
  dateRangeCurrencyData.xts <- currencyData.xts[index(currencyData.xts) >= startDateTime & index(currencyData.xts) <= endDateTime]
  
  # Return the currency data as a date selected subset of the original full xts data ser.
  return (dateRangeCurrencyData.xts)
}

GetFXSpotData2 <- function(filePath,filename, shiftHours, fromDateTime, toDateTime)
  # Read and select data from a csv file for a time period
  # GetFXSpotData2 uses a different date format and shifts the time frame of the data 
  # by the specified number of hours
  # 
  #
  ###########################################################################
{
  # Create the full csv import file path and name which includes the currency symbol
  # and the time frame period of the imported data file.
  
  filename = paste(filePath, filename, sep = "")
  
  # Open and read the file.
  csvFileData <-lapply(Sys.glob(filename), read.csv,na.strings = "unknown")
  
  
  #Convert file csv data to a data frame.
  currencyData.df <- as.data.frame(csvFileData, stringAsFactors=FALSE)
  
  # Add headers.
  colHeadings <-  c('gmtTime', 'open', 'high', 'low', 'close', 'volume')
  names(currencyData.df) <- colHeadings
  
  
  # convert currency data to XTS
  # Extract datetimes as POSIXct date time from hourly currency Data
  dateTimes <- currencyData.df$gmtTime
  posixDateTimes <- as.POSIXct(dateTimes,format= "%d.%m.%Y %H:%M:%S")
  # Convert values and dates to an xts object
  currencyData.xts <- as.xts(cbind(currencyData.df['open'],currencyData.df['high'],
                                   currencyData.df['low'], currencyData.df['close'],
                                   currencyData.df['volume']),order.by = posixDateTimes)
  
  # convert shift Hrs to seconds
  timeSeconds<- (shiftHours * 3600)
  # Time shift the Futures data 
  timeShiftedCurrencyData.xts <- shift.time(currencyData.xts, n = timeSeconds)
  
  # convert start and end date times to POSIXct
  startDateTime <- as.POSIXct(fromDateTime,format= "%Y-%m-%d %H:%M:%S")
  endDateTime <- as.POSIXct(toDateTime,format= "%Y-%m-%d %H:%M:%S")
  
  # Select currency data for the date range
  dateRangeCurrencyData.xts <- timeShiftedCurrencyData.xts[index(timeShiftedCurrencyData.xts) >= startDateTime & index(timeShiftedCurrencyData.xts) <= endDateTime]
  
  # Return the currency data as a date selected subset of the original full xts data ser.
  return (dateRangeCurrencyData.xts)
}


GetFXFuturesData <- function(filePath, filename)
  #
  # This function loads a csv formatted data file that has been exported from Metatrader.
  # Input parameters
  #   filepath - A character string that defines the full path to the directory
  #                where the data files resides. 
  #   filename -  Name of the FX futures  data file.
  #
  #  Returns - a data frame containing the following data items.
  #              datetime, bidPrice, nbrContracts. 
  #
  ###############################################################################
{
  # Create the full csv import file path and name which includes the currency symbol
  # and the time frame period of the imported data file.
  
  filename = paste(filePath, filename, sep = "")
  
  # Open and read the file.
  csvFileData <-lapply(Sys.glob(filename), read.csv2, na.strings = "unknown")
  
  #Convert file csv data to a data frame.
  data.df <- as.data.frame(csvFileData)
  
  # Add headers.
  colHeadings <-c('dateTime', 'ask', 'bid', 'last', 'volume')
  names(data.df) <- colHeadings
  
  # Extract and format the date time field
  date <- substr(data.df$dateTime,1,8)
  # insert "-" into date string
  formattedDate <- InsertIntoString(x=date, pos = 4, insert = "-") 
  formattedDate <- InsertIntoString(x=formattedDate, pos = 7, insert = "-") 
  # insert ":" into time string  
  time <- substr(data.df$dateTime,10,15)
  formattedTime <- InsertIntoString(x=time, pos = 2, insert = ":")
  formattedTime <- InsertIntoString(x=formattedTime, pos = 5, insert = ":")
  # join date and time into a datetime filed with a space between date and time
  formattedDateTime <- paste(formattedDate, formattedTime, sep=" ")
  # convert date time to POSIX format
  # posixDateTimes <- as.POSIXct(formattedDateTime,format= "%Y-%m-%d %H:%M:%S")
  posixDateTimes <- as.POSIXct(formattedDateTime,format= "%Y-%m-%d %H:%M:%S")
  
  # Futures data as data frame - Not currently used 
  # fxFutures.df <- as.data.frame(cbind(posixDateTimes, data.df['bid'],data.df['bid'],
  #                                      data.df['bid'],data.df['bid'],
  #                                    data.df['volume']))
  #  
  # Convery price and volume string values to numerics
  priceVal = as.numeric(data.df$bid)
  volumeVal = as.numeric(data.df$volume)
  
  # Create xts object setting Open,High,Low and Close to Bid value
  fxFutures.xts <- as.xts(cbind(priceVal,priceVal, priceVal, priceVal, volumeVal),
                          order.by = posixDateTimes)
  
  fxFutures.xts <- setNames(fxFutures.xts, c("Open", "High", "Low", "Close", "Volume"))
  
  #Remove seconds from xts data set
  dataSet <- fxFutures.xts
  index(dataSet) <- as.POSIXct(trunc(index(dataSet), units= "mins"))
  
  #Convert to 1 minute bars 
  fxFutures1MinBars.xts <- to.period(dataSet,"minutes",1)
  fxFutures1MinBars.xts <- setNames(fxFutures1MinBars.xts, c("Open", "High", "Low", "Close", "Volume"))
  
  return (fxFutures1MinBars.xts)
  
}



InsertIntoString<- function(x, pos, insert) 
  #  A function to insert a string or character into a target string
  #  Used to format date and time fields
  #
  ###########################################################################  
{   
  gsub(paste0("^(.{", pos, "})(.*)$"),
       paste0("\\1", insert, "\\2"),
       x)
}





######################
#
#      Main
#
######################

DataSet1 <- c(1,2,3,4,5,6,7,8,9,10)
DataSet2 <- c(2,4,6,8,10,12,14,16,18,20) # perfect positive correlation with DataSet1
DataSet3 <- c(-2,-4,-6,-8,-10,-12,-14,-16,-18,-20) #perfect negative correlation with DataSet1
DataSet4 <- c(2,7,12,2,14,5,18,12,2,10) # weak positive correlation with DataSet1
DataSet5 <- c(10,7,12,2,14,5,18,2,6,14) # close to zero positive correlation with DataSet1

##100% Correlated
##==============+
## Merge the two data sets into a dataframe with 2 columns
mergedDataSet1 <- data.frame(cbind(DataSet1, DataSet2))
# Use the Shapiro-Wilk test  to test for the normality of DataSet1. This helps determine the best correlation methid to use
shapiro.test(DataSet1) 
ggqqplot(mergedDataSet1, x ="DataSet1", ylab ="DataSet2", title = "Quantile plot of merged dataset 1")

ggscatter(mergedDataSet1,  x =  "DataSet1", y = "DataSet2", title = "100% Correlated",
          add = "reg.line", conf.int = TRUE, 
          cor.coef = TRUE, cor.method = "kendal",
          xlab = "DataSet 1", ylab = "DataSet 2")
corr1 <- cor.test(DataSet1, DataSet2,  method = "kendal")
print(corr1)


##100% Negative Correlated
##========================
## Merge the two data sets into a dataframe with 2 columns
mergedDataSet2 <- data.frame(cbind(DataSet1, DataSet3))
# Shapiro-Wilk normality test for DataSet3
shapiro.test(DataSet3) 
ggqqplot(mergedDataSet2, x ="DataSet1", ylab ="DataSet3", title = "Quantile plot of merged dataset 2")

ggscatter(mergedDataSet2,  x =  "DataSet1", y = "DataSet3", title = "100% Negatively Correlated",
          add = "reg.line", conf.int = TRUE, 
          cor.coef = TRUE, cor.method = "kendal",
          xlab = "DataSet 1", ylab = "DataSet 3")
corr2 <- cor.test(DataSet1, DataSet3,  method = "kendal")
print(corr2)  # Not used

##Weakly Correlated
##========================
## Merge the two data sets into a dataframe with 2 columns
mergedDataSet3<- data.frame(cbind(DataSet1, DataSet4))
# Shapiro-Wilk normality test for DataSet4
shapiro.test(DataSet4) 
ggqqplot(mergedDataSet3, x ="DataSet1", ylab ="DataSet4", title = "Quantile plot of merged dataset3")

ggscatter(mergedDataSet3,  x =  "DataSet1", y = "DataSet4", title = "Weak Correlation",
          add = "reg.line", conf.int = TRUE, 
          cor.coef = TRUE, cor.method = "pearson",
          xlab = "DataSet 1", ylab = "DataSet 4")
corr3 <- cor.test(DataSet1, DataSet4,  method = "pearson")
print(corr3)

##Almost No Correlation
##========================
## Merge the two data sets into a dataframe with 2 columns
mergedDataSet4<- data.frame(cbind(DataSet1, DataSet5))
# Shapiro-Wilk normality test for DataSet5
shapiro.test(DataSet5) 
ggqqplot(mergedDataSet3, x ="DataSet1", ylab ="DataSet5", title = "Quantile plot of merged dataset4")

ggscatter(mergedDataSet4,  x =  "DataSet1", y = "DataSet5", title = "Almost No Correlation",
          add = "reg.line", conf.int = TRUE, 
          cor.coef = TRUE, cor.method = "pearson",
          xlab = "DataSet 1", ylab = "DataSet 5")
corr4 <- cor.test(DataSet1, DataSet4,  method = "pearson")
print(corr4) # Not used


#  
# A Correlation Matrix
# Use to look at the correlations between multiple 
 mergedDataSet5 <- data.frame(cbind(DataSet1, DataSet2, DataSet3, DataSet4, DataSet5))
 cormatrix <- corrplot(cor(mergedDataSet5), title = "Correlations Between Multiple DataSets",
                       addCoefasPercent = TRUE, method = "pie", addCoef.col = "grey", mar=c(2,2,2,2))
 print(cormatrix)
 
 
 # Prepare the data as xts data sets
 #Pepperstone
 xtsFXSpotDataset1 <- GetFXSpotData(glvDataFilePath,glvFXSpotDataFile1, glvFromDateTime, glvToDateTime)
 #GoMarkets
 xtsFXSpotDataset2 <- GetFXSpotData(glvDataFilePath,glvFXSpotDataFile2, glvFromDateTime, glvToDateTime)
 #BlackBull Markets
 xtsFXSpotDataset3 <- GetFXSpotData(glvDataFilePath,glvFXSpotDataFile3, glvFromDateTime, glvToDateTime)
 #DukasCopy
 xtsFXSpotDataset4 <- GetFXSpotData2(glvDataFilePath,glvFXSpotDataFile4, glvDukasCopyShiftHrs,glvFromDateTime, glvToDateTime)
 
 
 ## Merge the three data sets into 1
 mergedDataSet <- merge(xtsFXSpotDataset1,xtsFXSpotDataset2, xtsFXSpotDataset3,xtsFXSpotDataset4)
 
 ## Simulate any missing data by by extrapolating between two nearset values
 cleanedDataSet <- na.approx(mergedDataSet)
 ##further clean the data by removing any rows with NA's'
 cleanedDataSet <- na.omit(cleanedDataSet)
 
 
 # Test for normality for price and volume   
 # Shapiro-Wilk normality test for dataItem1 close
 dataItem1<- coredata(cleanedDataSet$close)
 shapiro.test(dataItem1) 
 ggqqplot(xtsFXSpotDataset1, x ="close", ylab ="Close price")
 
 
 # Shapiro-Wilk normality test for dataItem1 volume
 dataItem3<- coredata(cleanedDataSet$volume.1)
 shapiro.test(dataItem3) 
 ggqqplot(xtsFXSpotDataset2, x ="volume", ylab ="volume")
 
 
 # price correlation and plot 
 #==========================
 ggscatter(cleanedDataSet,  x = "close", y = "close.1", title = "Price correlation GBPUSD Pepperstone & GoMarkets",
           add = "reg.line", conf.int = TRUE, 
           cor.coef = TRUE, cor.method = "pearson",
           xlab = "Pepperstone", ylab = "GoMarkets")
 
 
 ggscatter(cleanedDataSet,  x = "volume", y = "volume.1", title = "Volume correlation GBPUSD Pepperstone & GoMarkets",
           add = "reg.line", conf.int = TRUE, 
           cor.coef = TRUE, cor.method = "pearson",
           xlab = "Pepperstone", ylab = "GoMarkets")
 
 
 # Straight correlation test between Pepperstone and Go markets
 cor.test(cleanedDataSet$close, cleanedDataSet$close.1, method = "pearson")
 cor.test(cleanedDataSet$volume, cleanedDataSet$volume.1, method = "pearson")
 
 
 # Correlations Matrix for all 4 brokers 
 # Rename the closing price columns in the data so that they appear on the correlation chart
 PepperstonePrice <- cleanedDataSet$close
 colnames(PepperstonePrice)<- "PepperStone"
 GoMarketsPrice <- cleanedDataSet$close.1
 colnames(GoMarketsPrice) <- "GoMarkets"
 BlackBullPrice <- cleanedDataSet$close.2
 colnames(BlackBullPrice) <- "BlackBull"
 DukasCopyPrice <- cleanedDataSet$close.3
 colnames(DukasCopyPrice) <- "DukasCopy"
 
 # Create a price correlation matrix pie chart for the three brokers being evaluated
 mergedDataSet5 <- data.frame(cbind(PepperstonePrice, GoMarketsPrice, BlackBullPrice,DukasCopyPrice))
 corrplot(cor(mergedDataSet5), title = "GBPUSD Price Correlations\n 3 Brokers and 2nd Tier Market Maker/Bank",
          addCoefasPercent = TRUE, method = "pie", addCoef.col = "grey", mar=c(2,2,2,2))
 
 #individual price correlations
 cor.test(PepperstonePrice,DukasCopyPrice, method = "pearson")
 
 #Rename the volume columns in the data so that they appear on the correlation chart
 PepperstoneVolume <- cleanedDataSet$volume
 colnames(PepperstoneVolume)<- "PepperStone"
 GoMarketsVolume <- cleanedDataSet$volume.1
 colnames(GoMarketsVolume) <- "GoMarkets"
 BlackBullVolume <- cleanedDataSet$volume.2
 colnames(BlackBullVolume) <- "BlackBull"
 DukasCopyVolume <- cleanedDataSet$volume.3
 colnames(DukasCopyVolume) <- "DukasCopy"
 
 # Create a volume correlation matrix pie chart for the three brokers being evaluated
 mergedDataSet6 <- data.frame(cbind(PepperstoneVolume, GoMarketsVolume, BlackBullVolume,DukasCopyVolume))
 corrplot(cor(mergedDataSet6), title = "GBPUSD Volume Correlations\n 3 Brokers and 2nd Tier Market Maker/Bank",
          addCoefasPercent = TRUE, method = "pie", addCoef.col = "grey", mar=c(2,2,2,2))
 
 #individual volume correlations
 cor.test(PepperstoneVolume,DukasCopyVolume, method = "pearson")
 
 #individual price  or volume plots
 plot(PepperstonePrice)
 plot(DukasCopyPrice)
 plot(PepperstoneVolume)
 plot(DukasCopyVolume)
 
 
 FxFuturesData.xts <- GetFXFuturesData(glvDataFilePath,glvFXFuturesDataFile )
 # Adjust the time of the FX Futures data to match the FX Spot data and NewYork chart time (UTC +3)
 timeHrs <- 3
 # convert Hrs to seconds
 timeSeconds<- (timeHrs * 3600)
 # Time shift the Futures data 
 FxFuturesTimeShifted.xts <- shift.time(FxFuturesData.xts, n = timeSeconds)
 # Plot the futures price data
 pfp<- plot(FxFuturesTimeShifted.xts$Close, main = "FX Futures Price", lwd = 1 , col = "red", ylab = "Price")
 print(pfp)
 # plot the futures volume data 
 pfv<- plot(FxFuturesTimeShifted.xts$Volume, main = "FX Futures Contracts", lwd = 1 , col = "orange", ylab = "Nbr Contracts")
 print(pfv)
 # plot the spot price data 
 FxSpotData.xts <- GetFXSpotData(glvDataFilePath,glvFXSpotDataFile1, glvFromDateTime, glvToDateTime)
 psp <-plot.xts(FxSpotData.xts$close, main = "FX Spot Price", lwd = 1 , col = "blue", ylab = "Price") 
 print(psp)
 # plot the spot volume data
 psv<- plot(FxSpotData.xts$volume, main = "FX Spot Tick Volume", lwd = 1 , col = "orange", ylab = "Tick Volume")
 print(psv)
 
 
 ## Merge the two data sets into 1
 mergedDataSet <- merge(FxFuturesTimeShifted.xts, FxSpotData.xts)
 
 ## Remove rows with NA's
 cleanedDataSet2 <- na.omit(mergedDataSet)
 
 # #correlation and charting Price
 ggscatter(cleanedDataSet2,  x = "Close", y = "close",title ="FX Futures Price - Fx Spot Price",
           add = "reg.line", conf.int = TRUE,
           cor.coef = TRUE, cor.method = "pearson",
           xlab = "fx futures close price", ylab = "fx spot close price")
 
 
 # #correlation and charting Volume
 ggscatter(cleanedDataSet2,  x = "Volume", y = "volume",  title = "Fx Futures Contracts - Fx Tick Volume Correlation",
           add = "reg.line", conf.int = TRUE, 
           cor.coef = TRUE, cor.method = "pearson",
           xlab = "Nbr Futures Contracts", ylab = "Spot Tick Volume")
 
 
 # Create a volume correlation matrix pie chart for FX Futures and spot tick volume
 SpotTickVolume <- cleanedDataSet2$Volume
 colnames(SpotTickVolume)<- "FX_Futures_Contracts"
 FXFuturesContracts <- cleanedDataSet2$volume
 colnames(FXFuturesContracts)<- "Spot_Tick_Volume" 
 mergedDataSet <- data.frame(cbind(SpotTickVolume,FXFuturesContracts))
 corrplot(cor(mergedDataSet), title = "GBPUSD Tick Volume and FX Futures Contracts Correlation",
          addCoefasPercent = TRUE, method = "pie",  mar=c(2,2,2,2))
 
 
 # # Fx Futures Vs Spot price correlation
 res <- cor.test(cleanedDataSet2$Close, cleanedDataSet2$close,  method = "pearson")
 print(res)
 
 # # Fx Futures Contracts vs  spot tick Volume Correlation
 res2 <- cor.test(cleanedDataSet2$Volume, cleanedDataSet2$volume, method = "pearson")
 print(res2)
 
 #Brute force test for any leading or lagging relation between the futures and spot price data sets 
 # Initialize an empty dataframe to store results  
 cor.df <- data.frame(lag = integer(),correl=numeric()) 
 # loop through the data sets adjusting the lag each step            
 for (i in -10:10)
 {
   laggedFuturesData.xts <- lag(FxFuturesTimeShifted.xts, i)
   mergedLagTestDataSet <- merge(laggedFuturesData.xts, FxSpotData.xts) 
   ##clean the data by removing any rows with NA's'
   cleanedLagTestDataSet <- na.omit(mergedLagTestDataSet)
   ## Get correlation between the two sets of cleaned data
   res <- cor(cleanedLagTestDataSet$Close,cleanedLagTestDataSet$close,  method = "pearson")
   ## extract the value of the correlation coefficient
   corVal <- res[1]
   #Add the correlation coefficent to the summary dataframe 
   cor.df <- rbind(cor.df,c(i,corVal))
 }
 # plot the results
 plot(cor.df,main= "Cross Correlation - Fx Futures and Fx Spot Price",xlab = "Minutes Lag", ylab = "Cross Correlation")
 
  #spot = coredata(cleanedDataSet$Close.1)
 
 
 #Checking for leading /lagging relations between two time series using R's ccf function
 #============================================================================================================
 #First create a Posixct date time sequence with 1 minute intervals sufficient to hold all the elements of cleanedDataSet 
 nbrRows <- nrow(cleanedDataSet2)
 firstDateTime <- index(cleanedDataSet2[1])
 interval <- 60 #seconds
 lastDate <- firstDateTime + as.difftime(nbrRows-1, units="mins")
 dateTimeSeq <- seq(from=firstDateTime, by=interval*1, to=lastDate)
 #Use the contiguous dateTime sequence to generate a new Univariate data set
 cleanUnivariateDataSet.xts <- xts(cbind(cleanedDataSet2$Close, cleanedDataSet2$close),order.by = dateTimeSeq)
 #Extract xts Uivariate FxFutures price and FxSpot price time series 
 FxFuturesPrice = as.numeric(cleanUnivariateDataSet.xts$Close)
 FxSpotPrice = as.numeric(cleanUnivariateDataSet.xts$close)
 #Generate an auto correlated series using "R" cross correlation (covariance) function from the tseries library  
 acSeries<- ccf(FxFuturesPrice,FxSpotPrice,20,plot = FALSE)
 plot(acSeries, ylim = c(0.96,1), xlab= "Lag (minutes)")
 
 
